@@ -1,16 +1,16 @@
 # frozen_string_literal: true
 
 module ActionCable
-  # Returns the version of the currently loaded Action Cable as a <tt>Gem::Version</tt>.
+  # Returns the currently loaded version of Action Cable as a +Gem::Version+.
   def self.gem_version
     Gem::Version.new VERSION::STRING
   end
 
   module VERSION
-    MAJOR = 5
-    MINOR = 2
-    TINY  = 8
-    PRE   = nil
+    MAJOR = 7
+    MINOR = 1
+    TINY  = 5
+    PRE   = "2"
 
     STRING = [MAJOR, MINOR, TINY, PRE].compact.join(".")
   end
