@@ -19,15 +19,14 @@ module ActiveRecord
         @conn = Base.sqlite3_connection database: ":memory:",
                                         adapter: "sqlite3",
                                         timeout: 100
-
-        @connection_handler = ActiveRecord::Base.connection_handler
       end
 
       def test_bad_connection
-        assert_raise ActiveRecord::NoDatabaseError do
+        error = assert_raise ActiveRecord::NoDatabaseError do
           connection = ActiveRecord::Base.sqlite3_connection(adapter: "sqlite3", database: "/tmp/should/_not/_exist/-cinco-dog.db")
           connection.drop_table "ex", if_exists: true
         end
+        assert_kind_of ActiveRecord::ConnectionAdapters::NullPool, error.connection_pool
       end
 
       def test_database_exists_returns_false_when_the_database_does_not_exist
@@ -97,6 +96,19 @@ module ActiveRecord
         end
       end
 
+      def test_exec_insert_with_quote
+        with_example_table do
+          vals = [Relation::QueryAttribute.new("number", 10, Type::Value.new)]
+          @conn.exec_insert("insert into \"ex\" (number) VALUES (?)", "SQL", vals)
+
+          result = @conn.exec_query(
+            "select number from \"ex\" where number = ?", "SQL", vals)
+
+          assert_equal 1, result.rows.length
+          assert_equal 10, result.rows.first.first
+        end
+      end
+
       def test_primary_key_returns_nil_for_no_pk
         with_example_table "id int, data string" do
           assert_nil @conn.primary_key("ex")
@@ -105,16 +117,18 @@ module ActiveRecord
 
       def test_connection_no_db
         assert_raises(ArgumentError) do
-          Base.sqlite3_connection { }
+          Base.sqlite3_connection({})
         end
       end
 
       def test_bad_timeout
-        assert_raises(TypeError) do
-          Base.sqlite3_connection database: ":memory:",
+        exception = assert_raises(ActiveRecord::StatementInvalid) do
+          Base.sqlite3_connection(database: ":memory:",
                                   adapter: "sqlite3",
-                                  timeout: "usa"
+                                  timeout: "usa").connect!
         end
+        assert_match("TypeError", exception.message)
+        assert_kind_of ActiveRecord::ConnectionAdapters::NullPool, exception.connection_pool
       end
 
       # connection is OK with a nil timeout
@@ -122,6 +136,7 @@ module ActiveRecord
         conn = Base.sqlite3_connection database: ":memory:",
                                        adapter: "sqlite3",
                                        timeout: nil
+        conn.connect!
         assert conn, "made a connection"
       end
 
@@ -134,6 +149,26 @@ module ActiveRecord
         assert_equal "UTF-8", @conn.encoding
       end
 
+      def test_default_pragmas
+        if in_memory_db?
+          assert_equal [{ "foreign_keys" => 1 }], @conn.execute("PRAGMA foreign_keys")
+          assert_equal [{ "journal_mode" => "memory" }], @conn.execute("PRAGMA journal_mode")
+          assert_equal [{ "synchronous" => 2 }], @conn.execute("PRAGMA synchronous")
+          assert_equal [{ "journal_size_limit" => 67108864 }], @conn.execute("PRAGMA journal_size_limit")
+          assert_equal [], @conn.execute("PRAGMA mmap_size")
+          assert_equal [{ "cache_size" => 2000 }], @conn.execute("PRAGMA cache_size")
+        else
+          with_file_connection do |conn|
+            assert_equal [{ "foreign_keys" => 1 }], conn.execute("PRAGMA foreign_keys")
+            assert_equal [{ "journal_mode" => "wal" }], conn.execute("PRAGMA journal_mode")
+            assert_equal [{ "synchronous" => 1 }], conn.execute("PRAGMA synchronous")
+            assert_equal [{ "journal_size_limit" => 67108864 }], conn.execute("PRAGMA journal_size_limit")
+            assert_equal [{ "mmap_size" => 134217728 }], conn.execute("PRAGMA mmap_size")
+            assert_equal [{ "cache_size" => 2000 }], conn.execute("PRAGMA cache_size")
+          end
+        end
+      end
+
       def test_exec_no_binds
         with_example_table "id int, data string" do
           result = @conn.exec_query("SELECT id, data FROM ex")
@@ -141,7 +176,7 @@ module ActiveRecord
           assert_equal 2, result.columns.length
           assert_equal %w{ id data }, result.columns
 
-          @conn.exec_query('INSERT INTO ex (id, data) VALUES (1, "foo")')
+          @conn.exec_query("INSERT INTO ex (id, data) VALUES (1, 'foo')")
           result = @conn.exec_query("SELECT id, data FROM ex")
           assert_equal 1, result.rows.length
           assert_equal 2, result.columns.length
@@ -152,7 +187,7 @@ module ActiveRecord
 
       def test_exec_query_with_binds
         with_example_table "id int, data string" do
-          @conn.exec_query('INSERT INTO ex (id, data) VALUES (1, "foo")')
+          @conn.exec_query("INSERT INTO ex (id, data) VALUES (1, 'foo')")
           result = @conn.exec_query(
             "SELECT id, data FROM ex WHERE id = ?", nil, [Relation::QueryAttribute.new(nil, 1, Type::Value.new)])
 
@@ -165,7 +200,7 @@ module ActiveRecord
 
       def test_exec_query_typecasts_bind_vals
         with_example_table "id int, data string" do
-          @conn.exec_query('INSERT INTO ex (id, data) VALUES (1, "foo")')
+          @conn.exec_query("INSERT INTO ex (id, data) VALUES (1, 'foo')")
 
           result = @conn.exec_query(
             "SELECT id, data FROM ex WHERE id = ?", nil, [Relation::QueryAttribute.new("id", "1-fuu", Type::Integer.new)])
@@ -221,7 +256,12 @@ module ActiveRecord
         with_example_table do
           sql = "INSERT INTO ex (number) VALUES (10)"
           name = "foo"
-          assert_logged [[sql, name, []]] do
+
+          sqlite_version_query = ["SELECT sqlite_version(*)", "SCHEMA", []]
+          pragma_query = ["PRAGMA table_info(\"ex\")", "SCHEMA", []]
+          schema_query = ["SELECT sql FROM (SELECT * FROM sqlite_master UNION ALL SELECT * FROM sqlite_temp_master) WHERE type = 'table' AND name = 'ex'", "SCHEMA", []]
+          modified_insert_query = [(sql + ' RETURNING "id"'), name, []]
+          assert_logged [sqlite_version_query, pragma_query, schema_query, modified_insert_query] do
             @conn.insert(sql, name)
           end
         end
@@ -236,6 +276,32 @@ module ActiveRecord
         end
       end
 
+      def test_exec_insert_with_returning_disabled
+        original_conn = @conn
+        @conn = Base.sqlite3_connection database: ":memory:",
+                                       adapter: "sqlite3",
+                                       insert_returning: false
+        with_example_table do
+          result = @conn.exec_insert("insert into ex (number) VALUES ('foo')", nil, [], "id")
+          expect = @conn.query("select max(id) from ex").first.first
+          assert_equal expect.to_i, result.rows.first.first
+        end
+        @conn = original_conn
+      end
+
+      def test_exec_insert_default_values_with_returning_disabled
+        original_conn = @conn
+        @conn = Base.sqlite3_connection database: ":memory:",
+                                       adapter: "sqlite3",
+                                       insert_returning: false
+        with_example_table do
+          result = @conn.exec_insert("insert into ex DEFAULT VALUES", nil, [], "id")
+          expect = @conn.query("select max(id) from ex").first.first
+          assert_equal expect.to_i, result.rows.first.first
+        end
+        @conn = original_conn
+      end
+
       def test_select_rows
         with_example_table do
           2.times do |i|
@@ -282,6 +348,7 @@ module ActiveRecord
         sql = <<~SQL
           SELECT name FROM sqlite_master WHERE name <> 'sqlite_sequence' AND type IN ('table')
         SQL
+        @conn.connect!
         assert_logged [[sql.squish, "SCHEMA", []]] do
           @conn.tables
         end
@@ -441,44 +508,51 @@ module ActiveRecord
       end
 
       class Barcode < ActiveRecord::Base
+      end
+
+      class BarcodeCustomPk < ActiveRecord::Base
         self.primary_key = "code"
       end
 
       def test_copy_table_with_existing_records_have_custom_primary_key
-        connection = Barcode.connection
-        connection.create_table(:barcodes, primary_key: "code", id: :string, limit: 42, force: true) do |t|
+        connection = BarcodeCustomPk.connection
+        connection.create_table(:barcode_custom_pks, primary_key: "code", id: :string, limit: 42, force: true) do |t|
           t.text :other_attr
         end
         code = "214fe0c2-dd47-46df-b53b-66090b3c1d40"
-        Barcode.create!(code: code, other_attr: "xxx")
+        BarcodeCustomPk.create!(code: code, other_attr: "xxx")
 
-        connection.remove_column("barcodes", "other_attr")
+        connection.remove_column("barcode_custom_pks", "other_attr")
 
-        assert_equal code, Barcode.first.id
+        assert_equal code, BarcodeCustomPk.first.id
       ensure
-        Barcode.reset_column_information
+        BarcodeCustomPk.reset_column_information
+      end
+
+      class BarcodeCpk < ActiveRecord::Base
+        self.primary_key = ["region", "code"]
       end
 
       def test_copy_table_with_composite_primary_keys
-        connection = Barcode.connection
-        connection.create_table(:barcodes, primary_key: ["region", "code"], force: true) do |t|
+        connection = BarcodeCpk.connection
+        connection.create_table(:barcode_cpks, primary_key: ["region", "code"], force: true) do |t|
           t.string :region
           t.string :code
           t.text :other_attr
         end
         region = "US"
         code = "214fe0c2-dd47-46df-b53b-66090b3c1d40"
-        Barcode.create!(region: region, code: code, other_attr: "xxx")
+        BarcodeCpk.create!(region: region, code: code, other_attr: "xxx")
 
-        connection.remove_column("barcodes", "other_attr")
+        connection.remove_column("barcode_cpks", "other_attr")
 
-        assert_equal ["region", "code"], connection.primary_keys("barcodes")
+        assert_equal ["region", "code"], connection.primary_keys("barcode_cpks")
 
-        barcode = Barcode.first
+        barcode = BarcodeCpk.first
         assert_equal region, barcode.region
         assert_equal code, barcode.code
       ensure
-        Barcode.reset_column_information
+        BarcodeCpk.reset_column_information
       end
 
       def test_custom_primary_key_in_create_table
@@ -560,6 +634,27 @@ module ActiveRecord
         Barcode.reset_column_information
       end
 
+      def test_auto_increment_preserved_on_table_changes
+        connection = Barcode.connection
+        connection.create_table :barcodes, force: true do |t|
+          t.string :code
+        end
+
+        pk_column = connection.columns("barcodes").find { |col| col.name == "id" }
+        sql = connection.exec_query("SELECT sql FROM sqlite_master WHERE tbl_name='barcodes'").rows.first.first
+
+        assert(pk_column.auto_increment?)
+        assert(sql.match?("PRIMARY KEY AUTOINCREMENT"))
+
+        connection.change_column(:barcodes, :code, :integer)
+
+        pk_column = connection.columns("barcodes").find { |col| col.name == "id" }
+        sql = connection.exec_query("SELECT sql FROM sqlite_master WHERE tbl_name='barcodes'").rows.first.first
+
+        assert(pk_column.auto_increment?)
+        assert(sql.match?("PRIMARY KEY AUTOINCREMENT"))
+      end
+
       def test_supports_extensions
         assert_not @conn.supports_extensions?, "does not support extensions"
       end
@@ -576,15 +671,18 @@ module ActiveRecord
         db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
         db = ::SQLite3::Database.new(db_config.database)
 
+        @conn.connect!
+
         statement = ::SQLite3::Statement.new(db,
                                            "CREATE TABLE statement_test (number integer not null)")
         statement.stub(:step, -> { raise ::SQLite3::BusyException.new("busy") }) do
           assert_called(statement, :columns, returns: []) do
             assert_called(statement, :close) do
               ::SQLite3::Statement.stub(:new, statement) do
-                assert_raises ActiveRecord::StatementInvalid do
+                error = assert_raises ActiveRecord::StatementInvalid do
                   @conn.exec_query "select * from statement_test"
                 end
+                assert_equal @conn.pool, error.connection_pool
               end
             end
           end
@@ -595,6 +693,7 @@ module ActiveRecord
         conn = Base.sqlite3_connection database: ":memory:",
                                        adapter: "sqlite3",
                                        readonly: false
+        conn.connect!
 
         assert_not_predicate conn.raw_connection, :readonly?
       end
@@ -602,6 +701,7 @@ module ActiveRecord
       def test_db_is_not_readonly_when_readonly_option_is_unspecified
         conn = Base.sqlite3_connection database: ":memory:",
                                        adapter: "sqlite3"
+        conn.connect!
 
         assert_not_predicate conn.raw_connection, :readonly?
       end
@@ -610,6 +710,7 @@ module ActiveRecord
         conn = Base.sqlite3_connection database: ":memory:",
                                        adapter: "sqlite3",
                                        readonly: true
+        conn.connect!
 
         assert_predicate conn.raw_connection, :readonly?
       end
@@ -618,10 +719,109 @@ module ActiveRecord
         conn = Base.sqlite3_connection database: ":memory:",
                                        adapter: "sqlite3",
                                        readonly: true
+        conn.connect!
 
-        assert_raises(ActiveRecord::StatementInvalid, /SQLite3::ReadOnlyException/) do
+        exception = assert_raises(ActiveRecord::StatementInvalid) do
           conn.execute("CREATE TABLE test(id integer)")
         end
+        assert_match("SQLite3::ReadOnlyException", exception.message)
+        assert_equal conn.pool, exception.connection_pool
+      end
+
+      def test_strict_strings_by_default
+        conn = Base.sqlite3_connection(database: ":memory:", adapter: "sqlite3")
+        conn.create_table :testings
+
+        assert_nothing_raised do
+          conn.add_index :testings, :non_existent
+        end
+
+        with_strict_strings_by_default do
+          conn = Base.sqlite3_connection(database: ":memory:", adapter: "sqlite3")
+          conn.create_table :testings
+
+          error = assert_raises(StandardError) do
+            conn.add_index :testings, :non_existent2
+          end
+          assert_match(/no such column: "?non_existent2"?/, error.message)
+          assert_equal conn.pool, error.connection_pool
+        end
+      end
+
+      def test_strict_strings_by_default_and_true_in_database_yml
+        conn = Base.sqlite3_connection(database: ":memory:", adapter: "sqlite3", strict: true)
+        conn.create_table :testings
+
+        error = assert_raises(StandardError) do
+          conn.add_index :testings, :non_existent
+        end
+        assert_match(/no such column: "?non_existent"?/, error.message)
+        assert_equal conn.pool, error.connection_pool
+
+        with_strict_strings_by_default do
+          conn = Base.sqlite3_connection(database: ":memory:", adapter: "sqlite3", strict: true)
+          conn.create_table :testings
+
+          error = assert_raises(StandardError) do
+            conn.add_index :testings, :non_existent2
+          end
+          assert_match(/no such column: "?non_existent2"?/, error.message)
+          assert_equal conn.pool, error.connection_pool
+        end
+      end
+
+      def test_strict_strings_by_default_and_false_in_database_yml
+        conn = Base.sqlite3_connection(database: ":memory:", adapter: "sqlite3", strict: false)
+        conn.create_table :testings
+
+        assert_nothing_raised do
+          conn.add_index :testings, :non_existent
+        end
+
+        with_strict_strings_by_default do
+          conn = Base.sqlite3_connection(database: ":memory:", adapter: "sqlite3", strict: false)
+          conn.create_table :testings
+
+          assert_nothing_raised do
+            conn.add_index :testings, :non_existent
+          end
+        end
+      end
+
+      def test_rowid_column
+        with_example_table "id_uppercase INTEGER PRIMARY KEY" do
+          assert @conn.columns("ex").index_by(&:name)["id_uppercase"].rowid
+        end
+      end
+
+      def test_lowercase_rowid_column
+        with_example_table "id_lowercase integer PRIMARY KEY" do
+          assert @conn.columns("ex").index_by(&:name)["id_lowercase"].rowid
+        end
+      end
+
+      def test_non_integer_column_returns_false_for_rowid
+        with_example_table "id_int_short int PRIMARY KEY" do
+          assert_not @conn.columns("ex").index_by(&:name)["id_int_short"].rowid
+        end
+      end
+
+      def test_mixed_case_integer_colum_returns_true_for_rowid
+        with_example_table "id_mixed_case InTeGeR PRIMARY KEY" do
+          assert @conn.columns("ex").index_by(&:name)["id_mixed_case"].rowid
+        end
+      end
+
+      def test_rowid_column_with_autoincrement_returns_true_for_rowid
+        with_example_table "id_autoincrement integer PRIMARY KEY AUTOINCREMENT" do
+          assert @conn.columns("ex").index_by(&:name)["id_autoincrement"].rowid
+        end
+      end
+
+      def test_integer_cpk_column_returns_false_for_rowid
+        with_example_table("id integer, shop_id integer, PRIMARY KEY (shop_id, id)", "cpk_table") do
+          assert_not @conn.columns("cpk_table").any?(&:rowid)
+        end
       end
 
       private
@@ -641,6 +841,24 @@ module ActiveRecord
           SQL
           super(@conn, table_name, definition, &block)
         end
+
+        def with_strict_strings_by_default
+          SQLite3Adapter.strict_strings_by_default = true
+          yield
+        ensure
+          SQLite3Adapter.strict_strings_by_default = false
+        end
+
+        def with_file_connection(options = {})
+          options = options.dup
+          db_config = ActiveRecord::Base.configurations.configurations.find { |config| !config.database.include?(":memory:") }
+          options[:database] ||= db_config.database
+          conn = ActiveRecord::Base.sqlite3_connection(options)
+
+          yield(conn)
+        ensure
+          conn.disconnect! if conn
+        end
     end
   end
 end
