@@ -4,6 +4,7 @@ require "cases/helper"
 require "models/person"
 require "models/topic"
 require "pp"
+require "models/cpk"
 
 class NonExistentTable < ActiveRecord::Base; end
 
@@ -36,6 +37,10 @@ class CoreTest < ActiveRecord::TestCase
     assert_match(/Topic id: nil/, Topic.new.inspect)
   end
 
+  def test_inspect_singleton_instance
+    assert_match(/#<Class:#<Topic:\w+>>/, Topic.new.singleton_class.inspect)
+  end
+
   def test_inspect_limited_select_instance
     assert_equal %(#<Topic id: 1>), Topic.all.merge!(select: "id", where: "id = 1").first.inspect
     assert_equal %(#<Topic id: 1, title: "The First Topic">), Topic.all.merge!(select: "id, title", where: "id = 1").first.inspect
@@ -50,6 +55,11 @@ class CoreTest < ActiveRecord::TestCase
     assert_equal "NonExistentTable(Table doesn't exist)", NonExistentTable.inspect
   end
 
+  def test_inspect_relation_with_virtual_field
+    relation = Topic.limit(1).select("1 as virtual_field")
+    assert_match(/virtual_field: 1/, relation.inspect)
+  end
+
   def test_pretty_print_new
     topic = Topic.new
     actual = +""
@@ -135,4 +145,53 @@ class CoreTest < ActiveRecord::TestCase
     PP.pp(topic, StringIO.new(actual))
     assert_match(/id: 1/, actual)
   end
+
+  def test_find_by_cache_does_not_duplicate_entries
+    Topic.initialize_find_by_cache
+    using_prepared_statements = Topic.connection.prepared_statements
+    topic_find_by_cache = Topic.instance_variable_get("@find_by_statement_cache")[using_prepared_statements]
+
+    assert_difference -> { topic_find_by_cache.size }, +1 do
+      Topic.find(1)
+    end
+    assert_no_difference -> { topic_find_by_cache.size } do
+      Topic.find_by(id: 1)
+    end
+  end
+
+  def test_composite_pk_models_added_to_a_set
+    library = Set.new
+    # with primary key present
+    library << Cpk::Book.new(id: [1, 2])
+
+    # duplicate
+    library << Cpk::Book.new(id: [1, 3])
+    library << Cpk::Book.new(id: [1, 3])
+
+    # without primary key being set
+    library << Cpk::Book.new(title: "Book A")
+    library << Cpk::Book.new(title: "Book B")
+
+    assert_equal 4, library.size
+  end
+
+  def test_composite_pk_models_equality
+    assert Cpk::Book.new(id: [1, 2]) == Cpk::Book.new(id: [1, 2])
+
+    assert_not Cpk::Book.new(id: [1, 2]) == Cpk::Book.new(id: [1, 3])
+    assert_not Cpk::Book.new == Cpk::Book.new
+    assert_not Cpk::Book.new(title: "Book A") == Cpk::Book.new(title: "Book B")
+    assert_not Cpk::Book.new(author_id: 1) == Cpk::Book.new(author_id: 1)
+    assert_not Cpk::Book.new(author_id: 1, title: "Same title") == Cpk::Book.new(author_id: 1, title: "Same title")
+  end
+
+  def test_composite_pk_models_hash
+    assert_equal Cpk::Book.new(id: [1, 2]).hash, Cpk::Book.new(id: [1, 2]).hash
+
+    assert_not_equal Cpk::Book.new(id: [1, 2]).hash, Cpk::Book.new(id: [1, 3]).hash
+    assert_not_equal Cpk::Book.new.hash, Cpk::Book.new.hash
+    assert_not_equal Cpk::Book.new(title: "Book A").hash, Cpk::Book.new(title: "Book B").hash
+    assert_not_equal Cpk::Book.new(author_id: 1).hash, Cpk::Book.new(author_id: 1).hash
+    assert_not_equal Cpk::Book.new(author_id: 1, title: "Same title").hash, Cpk::Book.new(author_id: 1, title: "Same title").hash
+  end
 end
