@@ -40,6 +40,38 @@ class QueryCacheTest < ActiveRecord::TestCase
     super
   end
 
+  def test_execute_clear_cache
+    assert_cache :off
+
+    mw = middleware { |env|
+      Post.first
+      query_cache = ActiveRecord::Base.connection.query_cache
+      assert_equal 1, query_cache.length, query_cache.keys
+      Post.connection.execute("SELECT 1")
+      query_cache = ActiveRecord::Base.connection.query_cache
+      assert_equal 0, query_cache.length, query_cache.keys
+    }
+    mw.call({})
+
+    assert_cache :off
+  end
+
+  def test_exec_query_clear_cache
+    assert_cache :off
+
+    mw = middleware { |env|
+      Post.first
+      query_cache = ActiveRecord::Base.connection.query_cache
+      assert_equal 1, query_cache.length, query_cache.keys
+      Post.connection.exec_query("SELECT 1")
+      query_cache = ActiveRecord::Base.connection.query_cache
+      assert_equal 0, query_cache.length, query_cache.keys
+    }
+    mw.call({})
+
+    assert_cache :off
+  end
+
   def test_writes_should_always_clear_cache
     assert_cache :off
 
@@ -74,36 +106,6 @@ class QueryCacheTest < ActiveRecord::TestCase
     assert_cache :off
   end
 
-  def test_query_cache_is_applied_to_legacy_connections_in_all_handlers
-    old_value = ActiveRecord.legacy_connection_handling
-    ActiveRecord.legacy_connection_handling = true
-
-    assert_deprecated do
-      ActiveRecord::Base.connection_handlers = {
-        writing: ActiveRecord::Base.default_connection_handler,
-        reading: ActiveRecord::ConnectionAdapters::ConnectionHandler.new
-      }
-    end
-
-    ActiveRecord::Base.connected_to(role: :reading) do
-      db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
-      ActiveRecord::Base.establish_connection(db_config)
-    end
-
-    mw = middleware { |env|
-      reading_handler = ActiveRecord::Base.connection_handlers[:reading]
-      ro_pool = reading_handler.connection_pool_list
-      ro_conn = ro_pool.first.connection
-      assert_predicate ActiveRecord::Base.connection, :query_cache_enabled
-      assert_predicate ro_conn, :query_cache_enabled
-    }
-
-    mw.call({})
-  ensure
-    clean_up_legacy_connection_handlers
-    ActiveRecord.legacy_connection_handling = old_value
-  end
-
   def test_query_cache_is_applied_to_all_connections
     ActiveRecord::Base.connected_to(role: :reading) do
       db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
@@ -111,12 +113,9 @@ class QueryCacheTest < ActiveRecord::TestCase
     end
 
     mw = middleware { |env|
-      rw_conn = ActiveRecord::Base.connection_handler.connection_pool_list(:writing).first.connection
-      assert_predicate rw_conn, :query_cache_enabled
-
-      ro_conn = ActiveRecord::Base.connection_handler.connection_pool_list(:reading).first.connection
-      assert_predicate ActiveRecord::Base.connection, :query_cache_enabled
-      assert_predicate ro_conn, :query_cache_enabled
+      ActiveRecord::Base.connection_handler.connection_pool_list(:all).each do |pool|
+        assert_predicate pool.connection, :query_cache_enabled
+      end
     }
 
     mw.call({})
@@ -125,74 +124,6 @@ class QueryCacheTest < ActiveRecord::TestCase
   end
 
   if Process.respond_to?(:fork) && !in_memory_db?
-    def test_query_cache_with_multiple_handlers_and_forked_processes_legacy_handling
-      old_value = ActiveRecord.legacy_connection_handling
-      ActiveRecord.legacy_connection_handling = true
-
-      assert_deprecated do
-        ActiveRecord::Base.connection_handlers = {
-          writing: ActiveRecord::Base.default_connection_handler,
-          reading: ActiveRecord::ConnectionAdapters::ConnectionHandler.new
-        }
-      end
-
-      ActiveRecord::Base.connected_to(role: :reading) do
-        db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
-        ActiveRecord::Base.establish_connection(db_config)
-      end
-
-      rd, wr = IO.pipe
-      rd.binmode
-      wr.binmode
-
-      pid = fork {
-        rd.close
-        status = 0
-
-        middleware { |env|
-          begin
-            assert_cache :clean
-
-            # first request dirties cache
-            ActiveRecord::Base.connected_to(role: :reading) do
-              Post.first
-              assert_cache :dirty
-            end
-
-            # should clear the cache
-            Post.create!(title: "a new post", body: "and a body")
-
-            # fails because cache is still dirty
-            ActiveRecord::Base.connected_to(role: :reading) do
-              assert_cache :clean
-              Post.first
-            end
-
-          rescue Minitest::Assertion => e
-            wr.write Marshal.dump e
-            status = 1
-          end
-        }.call({})
-
-        wr.close
-        exit!(status)
-      }
-
-      wr.close
-
-      Process.waitpid pid
-      if !$?.success?
-        raise Marshal.load(rd.read)
-      else
-        assert_predicate $?, :success?
-      end
-
-      rd.close
-    ensure
-      clean_up_legacy_connection_handlers
-      ActiveRecord.legacy_connection_handling = old_value
-    end
-
     def test_query_cache_with_forked_processes
       ActiveRecord::Base.connected_to(role: :reading) do
         db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
@@ -262,7 +193,7 @@ class QueryCacheTest < ActiveRecord::TestCase
             t.datetime :ending
           end
         end
-        ActiveRecord::FixtureSet.create_fixtures(self.class.fixture_path, ["tasks"], {}, ActiveRecord::Base)
+        ActiveRecord::FixtureSet.create_fixtures(self.class.fixture_paths, ["tasks"], {}, ActiveRecord::Base)
       end
 
       ActiveRecord::Base.connection_pool.connections.each do |conn|
@@ -279,7 +210,7 @@ class QueryCacheTest < ActiveRecord::TestCase
         assert_cache :dirty
 
         thread_1_connection = ActiveRecord::Base.connection
-        ActiveRecord::Base.clear_active_connections!
+        ActiveRecord::Base.connection_handler.clear_active_connections!(:all)
         assert_cache :off, thread_1_connection
 
         started = Concurrent::Event.new
@@ -301,7 +232,7 @@ class QueryCacheTest < ActiveRecord::TestCase
             started.set
             checked.wait
 
-            ActiveRecord::Base.clear_active_connections!
+            ActiveRecord::Base.connection_handler.clear_active_connections!(:all)
           }.call({})
         }
 
@@ -537,15 +468,16 @@ class QueryCacheTest < ActiveRecord::TestCase
 
   def test_cache_is_available_when_using_a_not_connected_connection
     skip "In-Memory DB can't test for using a not connected connection" if in_memory_db?
-    db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary").dup
-    db_config.owner_name = "test2"
-    ActiveRecord::Base.connection_handler.establish_connection(db_config)
+    db_config = ActiveRecord::Base.connection_db_config
+    original_connection = ActiveRecord::Base.remove_connection
+
+    ActiveRecord::Base.establish_connection(db_config)
     assert_not_predicate Task, :connected?
 
     Task.cache do
       assert_queries(1) { Task.find(1); Task.find(1) }
     ensure
-      ActiveRecord::Base.connection_handler.remove_connection_pool(db_config.owner_name)
+      ActiveRecord::Base.establish_connection(original_connection)
     end
   end
 
@@ -622,19 +554,19 @@ class QueryCacheTest < ActiveRecord::TestCase
   end
 
   def test_query_cache_does_not_establish_connection_if_unconnected
-    ActiveRecord::Base.clear_active_connections!
-    assert_not ActiveRecord::Base.connection_handler.active_connections? # Double check they are cleared
+    ActiveRecord::Base.connection_handler.clear_active_connections!(:all)
+    assert_not ActiveRecord::Base.connection_handler.active_connections?(:all) # Double check they are cleared
 
     middleware {
-      assert_not ActiveRecord::Base.connection_handler.active_connections?, "QueryCache forced ActiveRecord::Base to establish a connection in setup"
+      assert_not ActiveRecord::Base.connection_handler.active_connections?(:all), "QueryCache forced ActiveRecord::Base to establish a connection in setup"
     }.call({})
 
-    assert_not ActiveRecord::Base.connection_handler.active_connections?, "QueryCache forced ActiveRecord::Base to establish a connection in cleanup"
+    assert_not ActiveRecord::Base.connection_handler.active_connections?(:all), "QueryCache forced ActiveRecord::Base to establish a connection in cleanup"
   end
 
   def test_query_cache_is_enabled_on_connections_established_after_middleware_runs
-    ActiveRecord::Base.clear_active_connections!
-    assert_not ActiveRecord::Base.connection_handler.active_connections? # Double check they are cleared
+    ActiveRecord::Base.connection_handler.clear_active_connections!(:all)
+    assert_not ActiveRecord::Base.connection_handler.active_connections?(:all) # Double check they are cleared
 
     middleware {
       assert_predicate ActiveRecord::Base.connection, :query_cache_enabled
@@ -643,7 +575,7 @@ class QueryCacheTest < ActiveRecord::TestCase
   end
 
   def test_query_caching_is_local_to_the_current_thread
-    ActiveRecord::Base.clear_active_connections!
+    ActiveRecord::Base.connection_handler.clear_active_connections!(:all)
 
     middleware {
       assert ActiveRecord::Base.connection_pool.query_cache_enabled
@@ -652,95 +584,53 @@ class QueryCacheTest < ActiveRecord::TestCase
       Thread.new {
         assert_not ActiveRecord::Base.connection_pool.query_cache_enabled
         assert_not ActiveRecord::Base.connection.query_cache_enabled
+
+        ActiveRecord::Base.connection_handler.clear_active_connections!(:all)
       }.join
     }.call({})
   end
 
   def test_query_cache_is_enabled_on_all_connection_pools
     middleware {
-      ActiveRecord::Base.connection_handler.connection_pool_list.each do |pool|
+      ActiveRecord::Base.connection_handler.connection_pool_list(:all).each do |pool|
         assert pool.query_cache_enabled
         assert pool.connection.query_cache_enabled
       end
     }.call({})
   end
 
-  def test_clear_query_cache_is_called_on_all_legacy_connections
-    skip "with in memory db, reading role won't be able to see database on writing role" if in_memory_db?
-    old_value = ActiveRecord.legacy_connection_handling
-    ActiveRecord.legacy_connection_handling = true
+  # with in memory db, reading role won't be able to see database on writing role
+  unless in_memory_db?
+    def test_clear_query_cache_is_called_on_all_connections
+      ActiveRecord::Base.connected_to(role: :reading) do
+        db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
+        ActiveRecord::Base.establish_connection(db_config)
+      end
 
-    assert_deprecated do
-      ActiveRecord::Base.connection_handlers = {
-        writing: ActiveRecord::Base.default_connection_handler,
-        reading: ActiveRecord::ConnectionAdapters::ConnectionHandler.new
+      mw = middleware { |env|
+        ActiveRecord::Base.connected_to(role: :reading) do
+          @topic = Topic.first
+        end
+
+        assert @topic
+
+        ActiveRecord::Base.connected_to(role: :writing) do
+          @topic.title = "Topic title"
+          @topic.save!
+        end
+
+        assert_equal "Topic title", @topic.title
+
+        ActiveRecord::Base.connected_to(role: :reading) do
+          @topic = Topic.first
+          assert_equal "Topic title", @topic.title
+        end
       }
+
+      mw.call({})
+    ensure
+      clean_up_connection_handler
     end
-
-    ActiveRecord::Base.connected_to(role: :reading) do
-      db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
-      ActiveRecord::Base.establish_connection(db_config)
-    end
-
-    mw = middleware { |env|
-      ActiveRecord::Base.connected_to(role: :reading) do
-        @topic = Topic.first
-      end
-
-      assert @topic
-
-      ActiveRecord::Base.connected_to(role: :writing) do
-        @topic.title = "Topic title"
-        @topic.save!
-      end
-
-      assert_equal "Topic title", @topic.title
-
-      ActiveRecord::Base.connected_to(role: :reading) do
-        @topic = Topic.first
-        assert_equal "Topic title", @topic.title
-      end
-    }
-
-    mw.call({})
-  ensure
-    unless in_memory_db?
-      clean_up_legacy_connection_handlers
-      ActiveRecord.legacy_connection_handling = old_value
-    end
-  end
-
-  def test_clear_query_cache_is_called_on_all_connections
-    skip "with in memory db, reading role won't be able to see database on writing role" if in_memory_db?
-
-    ActiveRecord::Base.connected_to(role: :reading) do
-      db_config = ActiveRecord::Base.configurations.configs_for(env_name: "arunit", name: "primary")
-      ActiveRecord::Base.establish_connection(db_config)
-    end
-
-    mw = middleware { |env|
-      ActiveRecord::Base.connected_to(role: :reading) do
-        @topic = Topic.first
-      end
-
-      assert @topic
-
-      ActiveRecord::Base.connected_to(role: :writing) do
-        @topic.title = "Topic title"
-        @topic.save!
-      end
-
-      assert_equal "Topic title", @topic.title
-
-      ActiveRecord::Base.connected_to(role: :reading) do
-        @topic = Topic.first
-        assert_equal "Topic title", @topic.title
-      end
-    }
-
-    mw.call({})
-  ensure
-    clean_up_connection_handler
   end
 
   test "query cache is enabled in threads with shared connection" do
@@ -760,9 +650,40 @@ class QueryCacheTest < ActiveRecord::TestCase
     ActiveRecord::Base.connection_pool.lock_thread = false
   end
 
+  test "query cache callbacks exit gracefully from a fork" do
+    # Regression test for #51298
+    pid = nil
+    status = nil
+
+    # Run forking Rack app
+    middleware { |env|
+      pid = fork
+      _, status = Process.wait2(pid) if pid.present?
+      [200, {}, nil]
+    }.call({})
+
+
+    # Happy path, the fork exit 0 and the main process
+    # asserts on the exit status of the fork
+    if pid.nil?
+      exit 0
+    else
+      assert_equal 0, status.exitstatus
+    end
+  rescue
+    # Unhappy path, the fork exit 1.
+    # The main process should not reach here, but in case
+    # something's wrong we need to re-raise the error.
+    if pid.nil?
+      exit 1
+    else
+      raise
+    end
+  end
+
   private
     def with_temporary_connection_pool(&block)
-      pool_config = ActiveRecord::Base.connection_handler.send(:owner_to_pool_manager).fetch("ActiveRecord::Base").get_pool_config(ActiveRecord.writing_role, :default)
+      pool_config = ActiveRecord::Base.connection.pool.pool_config
       new_pool = ActiveRecord::ConnectionAdapters::ConnectionPool.new(pool_config)
 
       pool_config.stub(:pool, new_pool, &block)
@@ -800,7 +721,7 @@ class QueryCacheMutableParamTest < ActiveRecord::TestCase
     attribute :payload, :json
   end
 
-  class HashWithFixedHash < Hash
+  class ObjectFixedHash < Struct.new(:a, :b)
     # this isn't very realistic, but it is the worst case and therefore a good
     # case to test
     def hash
@@ -821,12 +742,12 @@ class QueryCacheMutableParamTest < ActiveRecord::TestCase
   end
 
   def test_query_cache_handles_mutated_binds
-    JsonObj.create(payload: { a: 1 })
+    JsonObj.create(payload: ObjectFixedHash.new({ a: 1 }))
 
-    search = HashWithFixedHash[a: 1]
+    search = ObjectFixedHash.new({ a: 1 })
     JsonObj.where(payload: search).first # populate the cache
 
-    search.merge!(b: 2)
+    search.b = 2
     assert_nil JsonObj.where(payload: search).first, "cache returned a false positive"
   end
 
@@ -1007,6 +928,35 @@ class QueryCacheExpiryTest < ActiveRecord::TestCase
     end
   end
 
+  def test_query_cache_lru_eviction
+    connection = Post.connection
+    connection.pool.db_config.stub(:query_cache, 2) do
+      connection.send(:configure_query_cache!)
+      Post.cache do
+        assert_queries(2) do
+          connection.select_all("SELECT 1")
+          connection.select_all("SELECT 2")
+          connection.select_all("SELECT 1")
+        end
+
+        assert_queries(1) do
+          connection.select_all("SELECT 3")
+          connection.select_all("SELECT 3")
+        end
+
+        assert_no_queries do
+          connection.select_all("SELECT 1")
+        end
+
+        assert_queries(1) do
+          connection.select_all("SELECT 2")
+        end
+      end
+    end
+  ensure
+    connection.send(:configure_query_cache!)
+  end
+
   test "threads use the same connection" do
     @connection_1 = ActiveRecord::Base.connection.object_id
 
