@@ -2,8 +2,9 @@
 
 class Book < ActiveRecord::Base
   belongs_to :author
+  belongs_to :format_record, polymorphic: true
 
-  has_many :citations, foreign_key: "book1_id", inverse_of: :book
+  has_many :citations, inverse_of: :book
   has_many :references, -> { distinct }, through: :citations, source: :reference_of
 
   has_many :subscriptions
@@ -11,6 +12,8 @@ class Book < ActiveRecord::Base
 
   has_one :essay
 
+  alias_attribute :title, :name
+
   enum status: [:proposed, :written, :published]
   enum last_read: { unread: 0, reading: 2, read: 3, forgotten: nil }
   enum nullable_status: [:single, :married]
