@@ -1,5 +1,7 @@
 # frozen_string_literal: true
 
+require "active_support/testing/strict_warnings"
+
 ENV["RAILS_ENV"] ||= "test"
 require_relative "dummy/config/environment.rb"
 
@@ -8,40 +10,12 @@ require "active_support"
 require "active_support/test_case"
 require "active_support/core_ext/object/try"
 require "active_support/testing/autorun"
-require "active_support/configuration_file"
-require "active_storage/service/mirror_service"
 require "image_processing/mini_magick"
 
 require "active_job"
 ActiveJob::Base.queue_adapter = :test
 ActiveJob::Base.logger = ActiveSupport::Logger.new(nil)
 
-SERVICE_CONFIGURATIONS = begin
-  ActiveSupport::ConfigurationFile.parse(File.expand_path("service/configurations.yml", __dir__)).deep_symbolize_keys
-rescue Errno::ENOENT
-  puts "Missing service configuration file in test/service/configurations.yml"
-  {}
-end
-# Azure service tests are currently failing on the main branch.
-# We temporarily disable them while we get things working again.
-if ENV["CI"]
-  SERVICE_CONFIGURATIONS.delete(:azure)
-  SERVICE_CONFIGURATIONS.delete(:azure_public)
-end
-
-require "tmpdir"
-
-Rails.configuration.active_storage.service_configurations = SERVICE_CONFIGURATIONS.merge(
-  "local" => { "service" => "Disk", "root" => Dir.mktmpdir("active_storage_tests") },
-  "local_public" => { "service" => "Disk", "root" => Dir.mktmpdir("active_storage_tests"), "public" => true },
-  "disk_mirror_1" => { "service" => "Disk", "root" => Dir.mktmpdir("active_storage_tests_1") },
-  "disk_mirror_2" => { "service" => "Disk", "root" => Dir.mktmpdir("active_storage_tests_2") },
-  "disk_mirror_3" => { "service" => "Disk", "root" => Dir.mktmpdir("active_storage_tests_3") },
-  "mirror" => { "service" => "Mirror", "primary" => "local", "mirrors" => ["disk_mirror_1", "disk_mirror_2", "disk_mirror_3"] }
-).deep_stringify_keys
-
-Rails.configuration.active_storage.service = "local"
-
 ActiveStorage.logger = ActiveSupport::Logger.new(nil)
 ActiveStorage.verifier = ActiveSupport::MessageVerifier.new("Testing")
 ActiveStorage::FixtureSet.file_fixture_path = File.expand_path("fixtures/files", __dir__)
@@ -51,7 +25,7 @@ class ActiveSupport::TestCase
 
   include ActiveRecord::TestFixtures
 
-  self.fixture_path = File.expand_path("fixtures", __dir__)
+  self.fixture_paths = [File.expand_path("fixtures", __dir__)]
 
   setup do
     ActiveStorage::Current.url_options = { protocol: "https://", host: "example.com", port: nil }
@@ -61,16 +35,16 @@ class ActiveSupport::TestCase
     ActiveStorage::Current.reset
   end
 
-  def assert_queries(expected_count, &block)
+  def assert_queries(expected_count, matcher: nil, &block)
     ActiveRecord::Base.connection.materialize_transactions
 
     queries = []
     ActiveSupport::Notifications.subscribe("sql.active_record") do |*, payload|
-      queries << payload[:sql] unless %w[ SCHEMA TRANSACTION ].include?(payload[:name])
+      queries << payload[:sql] if %w[ SCHEMA TRANSACTION ].exclude?(payload[:name]) && (matcher.nil? || payload[:sql].match(matcher))
     end
 
     result = _assert_nothing_raised_or_warn("assert_queries", &block)
-    assert_equal expected_count, queries.size, "#{queries.size} instead of #{expected_count} queries were executed. #{queries.inspect}"
+    assert_equal expected_count, queries.size, "#{queries.size} instead of #{expected_count} queries were executed. Queries: #{queries.join("\n\n")}"
     result
   end
 
@@ -131,6 +105,7 @@ class ActiveSupport::TestCase
       strict_loading_was = ActiveRecord::Base.strict_loading_by_default
       ActiveRecord::Base.strict_loading_by_default = true
       yield
+    ensure
       ActiveRecord::Base.strict_loading_by_default = strict_loading_was
     end
 
@@ -138,6 +113,7 @@ class ActiveSupport::TestCase
       variant_tracking_was = ActiveStorage.track_variants
       ActiveStorage.track_variants = false
       yield
+    ensure
       ActiveStorage.track_variants = variant_tracking_was
     end
 
@@ -174,14 +150,38 @@ class User < ActiveRecord::Base
   has_one_attached :avatar_with_variants do |attachable|
     attachable.variant :thumb, resize_to_limit: [100, 100]
   end
+  has_one_attached :avatar_with_preprocessed do |attachable|
+    attachable.variant :bool, resize_to_limit: [1, 1], preprocessed: true
+  end
+  has_one_attached :avatar_with_conditional_preprocessed do |attachable|
+    attachable.variant :proc, resize_to_limit: [2, 2],
+      preprocessed: ->(user) { user.name == "transform via proc" }
+    attachable.variant :method, resize_to_limit: [3, 3],
+      preprocessed: :should_preprocessed?
+  end
+  has_one_attached :intro_video
+  has_one_attached :name_pronunciation_audio
 
   has_many_attached :highlights
   has_many_attached :vlogs, dependent: false, service: :local
   has_many_attached :highlights_with_variants do |attachable|
     attachable.variant :thumb, resize_to_limit: [100, 100]
   end
+  has_many_attached :highlights_with_preprocessed do |attachable|
+    attachable.variant :bool, resize_to_limit: [1, 1], preprocessed: true
+  end
+  has_many_attached :highlights_with_conditional_preprocessed do |attachable|
+    attachable.variant :proc, resize_to_limit: [2, 2],
+      preprocessed: ->(user) { user.name == "transform via proc" }
+    attachable.variant :method, resize_to_limit: [3, 3],
+      preprocessed: :should_preprocessed?
+  end
 
   accepts_nested_attributes_for :highlights_attachments, allow_destroy: true
+
+  def should_preprocessed?
+    name == "transform via method"
+  end
 end
 
 class Group < ActiveRecord::Base
