@@ -431,6 +431,36 @@ class HashWithIndifferentAccessTest < ActiveSupport::TestCase
     assert_equal(1, hash[:a])
     assert_equal(1, hash["a"])
     assert_instance_of ActiveSupport::HashWithIndifferentAccess, hash
+
+    hash = ActiveSupport::HashWithIndifferentAccess.new(@strings).transform_keys({ "a" => "x", "y" => "z" })
+
+    assert_nil(hash["a"])
+    assert_equal(1, hash["x"])
+    assert_equal(2, hash["b"])
+    assert_nil(hash["y"])
+    assert_nil(hash["z"])
+    assert_equal(["x", "b"], hash.keys) # asserting that order of keys is unchanged
+    assert_instance_of ActiveSupport::HashWithIndifferentAccess, hash
+
+    hash = ActiveSupport::HashWithIndifferentAccess.new(@strings).transform_keys({ "a" => "A", "q" => "Q" }) { |k| k * 3 }
+
+    assert_nil(hash["a"])
+    assert_equal(1, hash["A"])
+    assert_equal(2, hash["bbb"])
+    assert_nil(hash["q"])
+    assert_nil(hash["Q"])
+    assert_equal(["A", "bbb"], hash.keys) # asserting that order of keys is unchanged
+    assert_instance_of ActiveSupport::HashWithIndifferentAccess, hash
+
+    if RUBY_VERSION < "3"
+      assert_raise ArgumentError do
+        hash.transform_keys(nil)
+      end
+    else
+      assert_raise TypeError do
+        hash.transform_keys(nil)
+      end
+    end
   end
 
   def test_indifferent_deep_transform_keys
@@ -459,6 +489,38 @@ class HashWithIndifferentAccessTest < ActiveSupport::TestCase
     assert_equal(1, indifferent_strings[:a])
     assert_equal(1, indifferent_strings["a"])
     assert_instance_of ActiveSupport::HashWithIndifferentAccess, indifferent_strings
+
+    hash = ActiveSupport::HashWithIndifferentAccess.new(@strings)
+    hash.transform_keys!({ "a" => "x", "y" => "z" })
+
+    assert_nil(hash["a"])
+    assert_equal(1, hash["x"])
+    assert_equal(2, hash["b"])
+    assert_nil(hash["y"])
+    assert_nil(hash["z"])
+    assert_equal(["x", "b"], hash.keys) # asserting that order of keys is unchanged
+    assert_instance_of ActiveSupport::HashWithIndifferentAccess, hash
+
+    hash = ActiveSupport::HashWithIndifferentAccess.new(@strings)
+    hash.transform_keys!({ "a" => "A", "q" => "Q" }) { |k| k * 3 }
+
+    assert_nil(hash["a"])
+    assert_equal(1, hash["A"])
+    assert_equal(2, hash["bbb"])
+    assert_nil(hash["q"])
+    assert_nil(hash["Q"])
+    assert_equal(["A", "bbb"], hash.keys) # asserting that order of keys is unchanged
+    assert_instance_of ActiveSupport::HashWithIndifferentAccess, hash
+
+    if RUBY_VERSION < "3"
+      assert_raise ArgumentError do
+        hash.transform_keys(nil)
+      end
+    else
+      assert_raise TypeError do
+        hash.transform_keys(nil)
+      end
+    end
   end
 
   def test_indifferent_deep_transform_keys_bang
@@ -817,13 +879,8 @@ class HashWithIndifferentAccessTest < ActiveSupport::TestCase
 
     yaml_output = klass.new.to_yaml
 
-    # `hash-with-ivars` was introduced in 2.0.9 (https://git.io/vyUQW)
-    if Gem::Version.new(Psych::VERSION) >= Gem::Version.new("2.0.9")
-      assert_includes yaml_output, "hash-with-ivars"
-      assert_includes yaml_output, "@foo: bar"
-    else
-      assert_includes yaml_output, "hash"
-    end
+    assert_includes yaml_output, "hash-with-ivars"
+    assert_includes yaml_output, "@foo: bar"
   end
 
   def test_should_use_default_proc_for_unknown_key
